# =============================================================================
# Minet Twitter Tweet-date CLI Action
# =============================================================================
#
# Logic of the `tw tweet-date` action.
#
import re
import casanova

from minet.cli.utils import LoadingBar
from twitwi.utils import get_dates_from_id
from ural.twitter import parse_twitter_url, TwitterTweet, TwitterUser, TwitterList

ID_RE = re.compile(r"^[0-9]+$")


def twitter_tweet_date_action(cli_args):

    enricher = casanova.enricher(
        cli_args.file,
        cli_args.output,
        keep=cli_args.select,
        add=["timestamp_utc", "locale_time"],
    )

    loading_bar = LoadingBar("Getting tweets timestamp and date", unit="tweet")

    for row, tweet in enricher.cells(cli_args.column, with_rows=True):
        loading_bar.update()

        tweet_id = tweet

        tweet_parsed = parse_twitter_url(tweet)
        if isinstance(tweet_parsed, TwitterTweet):
            tweet_id = tweet_parsed.id
        elif isinstance(tweet_parsed, (TwitterUser, TwitterList)) or (
            not tweet_parsed and not ID_RE.match(tweet)
        ):
            loading_bar.inc("errors")
            loading_bar.print("%s is not a tweet id or url." % tweet)
            continue

        try:
            result = get_dates_from_id(tweet_id, locale=cli_args.timezone)

        except (ValueError, TypeError):
            loading_bar.inc("errors")
            enricher.writerow(row)

            continue

        enricher.writerow(row, list(result))



# =============================================================================
# Minet Instagram User-followings CLI Action
# =============================================================================
#
# Logic of the `instagram user-followings` action.
#

import casanova
from itertools import islice

from minet.constants import COOKIE_BROWSERS
from minet.cli.utils import LoadingBar, die
from minet.instagram import InstagramAPIScraper
from minet.instagram.constants import INSTAGRAM_USER_CSV_HEADERS
from minet.instagram.exceptions import (
    InstagramInvalidTargetError,
    InstagramInvalidCookieError,
    InstagramPrivateAccountError,
    InstagramAccountNoFollowError,
)


def user_following_action(cli_args):
    try:
        client = InstagramAPIScraper(cookie=cli_args.cookie)
    except InstagramInvalidCookieError:
        if cli_args.cookie in COOKIE_BROWSERS:
            die(['Could not extract relevant cookie from "%s".' % cli_args.cookie])

        die(
            [
                "Relevant cookie not found.",
                "A Facebook authentication cookie is necessary to be able to scrape Instagram.",
                "Use the --cookie flag to choose a browser from which to extract the cookie or give your cookie directly.",
            ]
        )

    enricher = casanova.enricher(
        cli_args.file,
        cli_args.output,
        add=INSTAGRAM_USER_CSV_HEADERS,
    )

    loading_bar = LoadingBar(
        "Retrieving followings", unit="user", stats={"followings": 0}
    )

    for i, (row, user) in enumerate(enricher.cells(cli_args.column, with_rows=True)):
        loading_bar.update()

        try:
            generator = client.user_following(user)

            if cli_args.limit:
                generator = islice(generator, cli_args.limit)

            for post in generator:
                enricher.writerow(row, post.as_csv_row())

                loading_bar.inc("followings")

        except InstagramInvalidTargetError:
            loading_bar.print(
                "Given user (line %i) is probably not an Instagram user: %s" % (i, user)
            )
        except InstagramAccountNoFollowError:
            loading_bar.print(
                "Given user (line %i) probably doesn't follow any account: %s"
                % (i, user)
            )
        except InstagramPrivateAccountError as nb_follow:
            loading_bar.print(
                "Given user (line %i) is probably a private account following %s accounts: %s"
                % (i, nb_follow, user)
            )



# =============================================================================
# Minet Telegram Channel-Infos CLI Action
# =============================================================================
#
# Action retrieving the information of a Telegram channel.
#
import casanova

from minet.cli.utils import LoadingBar
from minet.telegram import TelegramScraper
from minet.telegram.constants import TELEGRAM_INFOS_CSV_HEADERS
from minet.telegram.exceptions import TelegramInvalidTargetError


def channel_infos_action(cli_args):
    scraper = TelegramScraper(throttle=cli_args.throttle)
    enricher = casanova.enricher(
        cli_args.file,
        cli_args.output,
        add=TELEGRAM_INFOS_CSV_HEADERS,
        keep=cli_args.select,
    )

    loading_bar = LoadingBar("Retrieving infos", unit="channel")

    for i, (row, channel) in enumerate(
        enricher.cells(cli_args.column, with_rows=True), 1
    ):
        loading_bar.inc("channels")

        try:
            infos = scraper.channel_infos(channel)

            enricher.writerow(row, infos)

            loading_bar.update()

        except TelegramInvalidTargetError:
            loading_bar.print(
                "%s (line %i) is not a telegram channel or url, or is not accessible."
                % (channel, i)
            )



# =============================================================================
# Minet Instagram Formatters
# =============================================================================
#
# Various data formatters for Instagram data.
#
from casanova import namedrecord
from ebbe import getpath

from minet.instagram.constants import (
    INSTAGRAM_HASHTAG_POST_CSV_HEADERS,
    INSTAGRAM_USER_POST_CSV_HEADERS,
    INSTAGRAM_MEDIA_TYPE,
    INSTAGRAM_USER_CSV_HEADERS,
)
from minet.instagram.utils import (
    extract_hashtags,
    extract_handles,
    short_code_to_url,
)
from minet.utils import timestamp_to_isoformat

INSTAGRAM_PATH_TO_MEDIA_IMAGE = ["image_versions2", "candidates", 0, "url"]
INSTAGRAM_PATH_TO_MEDIA_VIDEO = ["video_versions", 0, "url"]

InstagramHashtagPost = namedrecord(
    "InstagramHashtagPost",
    INSTAGRAM_HASHTAG_POST_CSV_HEADERS,
    boolean=[
        "comments_disabled",
    ],
    plural=["hashtags", "mentioned_names"],
)

InstagramUserPost = namedrecord(
    "InstagramUserPost",
    INSTAGRAM_USER_POST_CSV_HEADERS,
    boolean=["like_and_view_counts_disabled", "is_verified"],
    plural=[
        "medias_type",
        "medias_url",
        "usertags_medias",
        "mentioned_names",
        "hashtags",
    ],
)

InstagramUser = namedrecord(
    "InstagramUser",
    INSTAGRAM_USER_CSV_HEADERS,
    boolean=[
        "is_private",
        "is_verified",
        "has_anonymous_profile_picture",
        "has_highlight_reels",
        "is_favorite",
    ],
)


def get_usertags(item):
    users = getpath(item, ["usertags", "in"])
    usertags = set()
    if not users:
        return usertags
    for user in users:
        username = getpath(user, ["user", "username"])
        usertags.add(username)
    return usertags


def format_hashtag_post(item):

    text = getpath(item, ["edge_media_to_caption", "edges", 0, "node", "text"])

    hashtags = []
    mentioned_names = []

    if text:
        hashtags = extract_hashtags(text)
        mentioned_names = extract_handles(text)

    row = InstagramHashtagPost(
        item["id"],
        item["__typename"],
        item["shortcode"],
        short_code_to_url(item["shortcode"]),
        text,
        hashtags,
        mentioned_names,
        item["edge_liked_by"]["count"],
        item["comments_disabled"],
        item["edge_media_to_comment"]["count"],
        item["edge_media_preview_like"]["count"],
        item.get("video_view_count"),
        item["owner"]["id"],
        item["taken_at_timestamp"],
        timestamp_to_isoformat(item["taken_at_timestamp"]),
        item["accessibility_caption"],
        short_code_to_url(item["shortcode"]) + "media/?size=l",
    )

    return row


def format_user_post(item):

    media_type = INSTAGRAM_MEDIA_TYPE.get(item["media_type"])
    if media_type is None:
        media_type = item["media_type"]

    text = getpath(item, ["caption", "text"])

    hashtags = []
    mentioned_names = []

    if text:
        hashtags = extract_hashtags(text)
        mentioned_names = extract_handles(text)

    medias_url = []
    medias_type = []
    usertags_medias = set()

    if media_type == "GraphSidecar":
        carousel = getpath(item, ["carousel_media"])
        for media in carousel:
            medias_type.append(INSTAGRAM_MEDIA_TYPE.get(getpath(media, ["media_type"])))

            if medias_type[-1] == "GraphImage":
                medias_url.append(getpath(media, INSTAGRAM_PATH_TO_MEDIA_IMAGE))
                item_usertags = get_usertags(media)
                usertags_medias = usertags_medias.union(item_usertags)

            elif medias_type[-1] == "GraphVideo":
                medias_url.append(getpath(media, INSTAGRAM_PATH_TO_MEDIA_VIDEO))
                item_usertags = get_usertags(media)
                usertags_medias = usertags_medias.union(item_usertags)

    elif media_type == "GraphImage":
        medias_type.append("GraphImage")
        medias_url.append(getpath(item, INSTAGRAM_PATH_TO_MEDIA_IMAGE))
        item_usertags = get_usertags(item)
        usertags_medias = usertags_medias.union(item_usertags)

    elif media_type == "GraphVideo":
        medias_type.append("GraphVideo")
        medias_url.append(getpath(item, INSTAGRAM_PATH_TO_MEDIA_VIDEO))
        item_usertags = get_usertags(item)
        usertags_medias = usertags_medias.union(item_usertags)

    row = InstagramUserPost(
        getpath(item, ["user", "username"]),
        getpath(item, ["user", "full_name"]),
        getpath(item, ["user", "is_verified"]),
        item["id"],
        media_type,
        item["code"],
        short_code_to_url(item["code"]),
        text,
        hashtags,
        mentioned_names,
        item["like_and_view_counts_disabled"],
        item["like_count"],
        item["comment_count"],
        short_code_to_url(item["code"]) + "media/?size=l",
        medias_type,
        medias_url,
        usertags_medias,
        item["taken_at"],
        timestamp_to_isoformat(item["taken_at"]),
    )

    return row


def format_user(item):

    row = InstagramUser(
        item.get("pk"),
        item.get("username"),
        item.get("full_name"),
        item.get("is_private"),
        item.get("profile_pic_url"),
        item.get("profile_pic_id"),
        item.get("is_verified"),
        item.get("has_anonymous_profile_picture"),
        item.get("has_highlight_reels"),
        item.get("account_badges"),
        item.get("similar_user_id"),
        item.get("latest_reel_media"),
        item.get("is_favorite"),
    )

    return row



# =============================================================================
# Minet Instagram CLI Action
# =============================================================================
#
# Logic of the `insta` action.
#


def instagram_action(cli_args):

    if cli_args.insta_action == "hashtag":

        from minet.cli.instagram.hashtag import hashtag_action

        hashtag_action(cli_args)

    if cli_args.insta_action == "user-posts":

        from minet.cli.instagram.user_posts import user_posts_action

        user_posts_action(cli_args)

    if cli_args.insta_action == "user-followers":

        from minet.cli.instagram.user_followers import user_followers_action

        user_followers_action(cli_args)

    if cli_args.insta_action == "user-following":

        from minet.cli.instagram.user_following import user_following_action

        user_following_action(cli_args)



# =============================================================================
# Minet Telegram CLI Action
# =============================================================================
#
# Logic of the `tl` action.
#


def telegram_action(cli_args):

    if cli_args.tl_action == "channel-infos":

        from minet.cli.telegram.channel_infos import channel_infos_action

        channel_infos_action(cli_args)

    elif cli_args.tl_action == "channel-messages":

        from minet.cli.telegram.channel_messages import channel_messages_action

        channel_messages_action(cli_args)



# =============================================================================
# Minet Tiktok Search-videos CLI Action
# =============================================================================
#
# Logic of the `tiktok search-videos` action.
#
import casanova
from itertools import islice

from minet.cli.utils import LoadingBar
from minet.tiktok import TiktokAPIScraper
from minet.tiktok.constants import TIKTOK_VIDEO_CSV_HEADERS


def search_videos_action(cli_args):
    enricher = casanova.enricher(
        cli_args.file,
        cli_args.output,
        add=TIKTOK_VIDEO_CSV_HEADERS,
    )

    loading_bar = LoadingBar("Retrieving videos", unit="querie", stats={"videos": 0})

    client = TiktokAPIScraper(cookie=cli_args.cookie)
    for row, query in enricher.cells(cli_args.column, with_rows=True):
        loading_bar.update()

        generator = client.search_videos(query)

        if cli_args.limit:
            generator = islice(generator, cli_args.limit)

        for video in generator:
            enricher.writerow(row, video.as_csv_row())

            loading_bar.inc("videos")
